/*
 * IrBallEmu.ino
 * RoboCupJunior パルス変調規格
 * MODE-Aをエミュレートする。
 */

#define LED_PIN  13

#define R5_PIN  7 //  200Ω  赤黒茶
#define R4_PIN  6 //  180Ω  茶灰茶
#define R3_PIN  5 //  150Ω  茶緑茶
#define R2_PIN  4 //  130Ω  茶橙茶
#define R1_PIN  3 //  100Ω  茶黒茶

/*
以下のタイミング調整で  1pulse  25usecとなる。
ただし、パルス切り替え時には、多少時間がかかってしまう。
*/

// duty比 1/2
#define   HIGH_TIME  6
#define   LOW__TIME  7

void setup() {
  // Serial.begin(9600);
  pinMode(   LED_PIN, OUTPUT);

  pinMode(    R1_PIN, OUTPUT);
  pinMode(    R2_PIN, OUTPUT);
  pinMode(    R3_PIN, OUTPUT);
  pinMode(    R4_PIN, OUTPUT);
  pinMode(    R5_PIN, OUTPUT);

  // すべて、Offの状態
  digitalWrite( R1_PIN, LOW );
  digitalWrite( R2_PIN, LOW );
  digitalWrite( R3_PIN, LOW );
  digitalWrite( R4_PIN, LOW );

  digitalWrite( R5_PIN, LOW );

  digitalWrite(LED_PIN, LOW );
  /*
  RoboCupJunior パルス変調規格
  MODE-A

  1波形 833usec 1.2KHz
  最小パルス 40KHz

  1000000usec /  1200 Hz = 833.3333333333 usec
  1000000usec / 40000 Hz = 25 usec

  1pulse  25usec
  FULL  8 pulse
  1/ 4  4 pulse
  1/16  4 pulse
  1/64  4 pulse
  休止期間　333 usec
    */
}

void OnePulse(int IrPin, int HighTime, int Low_Time) {
  digitalWrite(IrPin, HIGH); delayMicroseconds(HighTime);
  digitalWrite(IrPin, LOW ); delayMicroseconds(Low_Time);
}

// the loop function runs over and over again forever
// forループを使用すると、タイミングがズレるので、
// ループの展開を行い、力ずくでOnOffを繰り返すようにした。
void loop() {
  digitalWrite(R5_PIN, HIGH); // 200Ω 赤黒茶
  // R1 100Ω 茶黒茶
  // Full power
  // 8 pulse
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 1 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 2 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 3 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 4 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 5 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 6 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 7 */
  OnePulse(R1_PIN, HIGH_TIME, LOW__TIME); /* 8 */

  // R2 130Ω 茶橙茶
  // 1/ 4 power
  // 4 pulse
  OnePulse(R2_PIN, HIGH_TIME, LOW__TIME); /* 1 */
  OnePulse(R2_PIN, HIGH_TIME, LOW__TIME); /* 2 */
  OnePulse(R2_PIN, HIGH_TIME, LOW__TIME); /* 3 */
  OnePulse(R2_PIN, HIGH_TIME, LOW__TIME); /* 4 */

  // R3 150Ω 茶緑茶
  // 1/16 power
  // 4 pulse
  OnePulse(R3_PIN, HIGH_TIME, LOW__TIME); /* 1 */
  OnePulse(R3_PIN, HIGH_TIME, LOW__TIME); /* 2 */
  OnePulse(R3_PIN, HIGH_TIME, LOW__TIME); /* 3 */
  OnePulse(R3_PIN, HIGH_TIME, LOW__TIME); /* 4 */

  // R4 180Ω 茶灰茶
  // 1/64 power
  // 4 pulse
  OnePulse(R4_PIN, HIGH_TIME, LOW__TIME); /* 1 */
  OnePulse(R4_PIN, HIGH_TIME, LOW__TIME); /* 2 */
  OnePulse(R4_PIN, HIGH_TIME, LOW__TIME); /* 3 */
  OnePulse(R4_PIN, HIGH_TIME, LOW__TIME); /* 4 */

  digitalWrite(R5_PIN, LOW );


  // 休止期間　333 usec
  digitalWrite(LED_PIN, HIGH);
  delayMicroseconds(307);
  digitalWrite(LED_PIN, LOW );
}
